/*
** FILE NAME:	black.c
**
** PROJECT ID:	94-0040
**
** TITLE:	GenBlackman
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This file contains the routines used to generate the Blackman
**              window.
**
**		int GenBlackman(output, win_size)
**
** PARAMETERS:
**              short output[]       : array to contain window
**              long win_size        : size of window
**
** RETURNS:     EDSP_OK              : success
**              EDSP_BAD_ARG         : win_size < 2
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>
#define TWOPI 6.28318530717959
#define FOURPI 12.56637061435917

int
GenBlackman(short output[], long win_size)
{

    /* Variable declaration */
    int i, j;
    double wtemp2, wr2, wpi2, wpr2, wi2;
    double wtemp4, wr4, wpi4, wpr4, wi4;
    double arg2;
    double arg4;

    if (win_size <= 1)		/* Input Argument error checking */
	return EDSP_BAD_ARG;

    arg2 = TWOPI / win_size;	/* 2*pi / win_size */
    arg4 = FOURPI / win_size;	/* 4*pi / win_size */

    /* Variable initial value calculation */
    wtemp2 = sin(0.5 * arg2);
    wtemp4 = sin(0.5 * arg4);
    wpr2 = -2.0 * wtemp2 * wtemp2;
    wpr4 = -2.0 * wtemp4 * wtemp4;
    wpi2 = sin(arg2);
    wpi4 = sin(arg4);
    wr2 = 1.0;
    wr4 = 1.0;
    wi2 = 0.0;
    wi4 = 0.0;

    /* Test for even number of win_size */
    if ((win_size % 2) == 0)
	j = win_size / 2;
    else
	j = (win_size - 1) / 2;

    output[0] = floor(32767 * (0.42 - 0.5 * wr2 + 0.08 * wr4) + 0.5);

    for (i = 1; i <= j; i++)
    {
	/* Trig. recurence */
	wr2 = (wtemp2 = wr2) * wpr2 - wi2 * wpi2 + wr2;
	wi2 = wi2 * wpr2 + wtemp2 * wpi2 + wi2;
	wr4 = (wtemp4 = wr4) * wpr4 - wi4 * wpi4 + wr4;
	wi4 = wi4 * wpr4 + wtemp4 * wpi4 + wi4;

	output[i] = floor(32767 * (0.42 - 0.5 * wr2 + 0.08 * wr4) + 0.5);
	output[win_size - i] = output[i];

    }
    return EDSP_OK;		/* Return success  */

}
