;
; FILE NAME:    convpart.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        ConvPartial
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the partial convolution function.
;
;               int ConvPartial(output, iw, ix, iw_size, ix_size, res_shift)
;
; PARAMETERS:
;               short output[]: output y
;               short iw[]    : input w
;               short ix[]    : input x
;               long iw_size  : size of iw
;               long ix_size  : size of iw
;               int res_shift : right shift applied to each output
;

	.EXPORT	_ConvPartial
	.IMPORT	SHT

;REGISTER USAGE
;
;R1,R6 -> END OF X	;INPUT ARRAY
;R2,R5 -> W	;COEF
;R9 = MIN(IW_SIZE,IX_SIZE)
;R10 = COUNT
;R11 = TOTAL OUTPUT=IW_SIZE+IX_SIZE-1
;R4 -> OUTPUT
;R7 = IW_SIZE
;R12= IX_SIZE
;R13= IW_SIZE+IX_SIZE-R9
;R14= SHIFT

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_ConvPartial:
	MOV.L	@(0,R15),R1	;R1=IX_SIZE
	MOV.L	@(4,R15),R2	;R2=SHIFT
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV	R2,R14		;R14=SHIFT
	MOV.L	R11,@-R15
	MOV	R1,R12		;R12=IX_SIZE
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-8,R15

;ERROR CHECKING
	MOV	#27,R0
	CMP/GT	R0,R14		;SHIFT > 27 ?
	BT	RTN1
	CMP/PZ	R14		;SHIFT < 0 ?
	BF	RTN1
	CMP/PL	R7		;IW_SIZE > 0 ?
	BF	RTN1
	CMP/PL	R12		;IX_SIZE > 0 ? 
	BF	RTN1
	CMP/GE	R7,R12		;IX_SIZE > IW_SIZE ?
	BT	argsOK

RTN1:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4
argsOK:	MOV	R12,R9		;R9=IX_SIZE=LEN2
	MOV	R7,R13		;R13=IW_SIZE=LEN1

;Here R9 > R13	;R9=LEN2, R13=LEN1, R6 -> LONG INPUT, R5 -> SHORT INPUT
	SUB	R13,R9		;R9=LEN2-LEN1
	ADD	#1,R9

	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	AND	R13,R11		;R11=LOWER 2 BITS
	MOV	LOWMASK,R12	;MASK AWAY LOWER 2 BITS
	AND	R13,R12		;R12=UPPER 30 BITS
	MOV.L	R11,@(0,R15)	;STORE COUNT ON STACK
	SHLL2	R14
	MOV.L	R12,@(4,R15)	;STORE COUNT ON STACK

; precalculate address of shift routine
	SHLL2	R14
	MOV	SHTADDR,R3	;R3=ADDRESS OF SHIFT ROUTINE
	SHLL	R14
	ADD	R14,R3		;R3=SHIFT ROUTINE ADDRESS

	SHLL	R13		;LEN1*2
	ADD	R13,R5
	ADD	#-2,R5		;R5 -> END OF SHORT

	SHLL	R9
	ADD	R4,R9		;R9=OUTPUT END ADDRESS

;				;R6 -> BEGINNING OF LONG
;				;R5 -> END OF SHORT
outer:	MOV	R6,R1		;R1 ->LHS OF LONG
	MOV	R5,R2		;R2=R5 -> END OF SHORT

	MOV.L	@(0,R15),R11	;READ COUNT FROM STACK
	MOV	R7,R0
	MOV.L	@(4,R15),R12	;READ COUNT FROM STACK

	SHLL	R0
	MOV	R2,R13
	SUB	R0,R13

	MOVA	LB15,R0		;R0=ADDRESS OF LB
	SHLL2	R11		;R11*4
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	CLRMAC

	.ALIGN	4
inner:	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
	MAC.W	@R1+,@R2+
	ADD	#-4,R2
LB15:	CMP/EQ	R13,R2
	BF	inner

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R3		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER
	ADD	#2,R6		;LONG POINTER +2
	CMP/EQ	R9,R4
	BF	outer

RTN3:	MOV	#0,R0		;RETURN CODE
RTN:	ADD	#8,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	MOV	@R15+,R14
	RTS
	NOP

	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END
