;
; FILE NAME:    corrcycl.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        CorrCyclic
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the cyclic correlate function.
;
;               int CorrCyclic(output, iw, ix, size, res_shift)
;
; PARAMETERS:
;               short output[]: output Y
;               short iw[]    : input W
;               short ix[]    : input X
;               long size     : size of iw and ix, N
;               int res_shift : right shift applied to each output
;

	.EXPORT	_CorrCyclic
	.IMPORT	SHT

;REGISTER USAGE
;
;R2,R6 -> END OF x
;R3,R5 -> w
;R9 = min(iw_size,ix_size)
;R10= COUNT of outputs to generate
;R11= total no. outputs=iw_size+ix_size-1
;R4 -> output
;R7 = N
;R12,R13=scratch registers
;R14= SHIFT
;R6 -> X

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_CorrCyclic:	MOV.L	@(0,R15),R1	;R1=SHIFT
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV	R1,R14		;R14=SHIFT
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15

; Error checking
	CMP/PL	R7		;SIZE > 0 ?
	BF	RTN1
	MOV	#27,R0
	CMP/GT	R0,R14		;SHIFT > 27 ?
	BT	RTN1
	CMP/PZ	R14		;SHIFT < 0 ?
	BT	argsOK

RTN1:	BRA	RTN
	MOV	#1,R0

	.ALIGN	4

;R9=N, R6 -> LONG (X), R5 -> SHORT (W)

argsOK:	MOV	R14,R11
	SHLL2	R11
	SHLL2	R11
	SHLL	R11
	MOV	SHTADDR,R9	;R2=ADDRESS OF SHIFT ROUTINE
	ADD	R11,R9		;R9=SHIFT JUMP ADDRESS
	MOV	R6,R3		;R3=R6  -> BEGINNING OF X

				;R7=N
	MOV	R7,R10		;R10=COUNT
outer:	CLRMAC
	MOV	R5,R2		;R2=R5 -> BEGINNING OF W
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	R6,R1		;R1 -> LHS OF X
	AND	R10,R11		;R11=LOWER 2 BITS
	MOV.L	R10,R12
	MOVA	LB5,R0		;R0=ADDRESS OF LB5
	SHLL	R12
	SHLL	R11		;R11*2
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	ADD	R2,R12		;R12=LAST VALUE OF R2

	.ALIGN	4
loop1:	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
LB5:	CMP/EQ	R12,R2
	BF	loop1

	MOV	R7,R1		;R1=N
	SUB	R10,R1		;R1=N-R10

	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	AND	R1,R11		;R11=LOWER 2 BITS

	MOV.L	R1,R12
	SHLL	R12
	MOV	R3,R1		;R1 -> BEGINNING OF X
	SHLL	R11		;R11*2
	MOVA	LB25,R0		;R0=ADDRESS OF LB25
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	ADD	R1,R12		;R12=LAST VALUE OF R1

	.ALIGN	4
loop2:	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
	MAC.W	@R1+,@R2+
LB25:	CMP/EQ	R12,R1
	BF	loop2

	STS	MACH,R13
	STS	MACL,R8

;CALL SHIFT ROUTINE
	JSR	@R9		;CALL SHIFT ROUTINE
	CLRT

	MOV.W	R8,@R4		;OUTPUT RESULT
	ADD	#2,R4		;INCREASE POINTER

	ADD	#-1,R10
	ADD	#2,R6
	CMP/PL	R10
	BT	outer

	MOV	#0,R0		;RETURN CODE
RTN:	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14



	.ALIGN	4
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC

	.END
