/*
** FILE NAME:   FFT.C
**
** PROJECT ID:  DSPLIB SH  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a sine wave, applies a Hamming window 
**              applies an FFT and then returns the solution in log format
**
** MODIFICATION HISTORY:
**
**              1.00    29-NOV-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"


#define NSAM     128              /* number of samples */
#define NSAML    NSAM*2
#define NSAMS    NSAM/2
#define TWOPI     6.283185307
#define MAXSIZE   NSAML

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{

        
        short   sinea1[NSAM], window[NSAM], imaga[NSAML], imagb[NSAML], ifft[NSAM], sinea[NSAM], sineb[NSAM], fftouta[NSAM], fftoutb[NSAM], fftoutc[NSAML], fftoutd[NSAML], outputa[NSAMS], outputb[NSAMS], outputai[NSAMS], outputbi[NSAMS];
        int     n, k, len, sine, res_shift;
        float   scale, scalefft;

        scale           =       1;
        res_shift       =       0;
        len             =       NSAM;
        scalefft        =       EFFTALLSCALE;
        sine            =       0; /*** Set To zero for sine, one for square***/


/*** Initiate Twiddle Values ***/


        if(InitFft (MAXSIZE) != EDSP_OK)
        printf("Problem With Initialisation Of FFT");

/*** Generate Square Wave ***/   

        for(n=0; n < len; n=n+12)
        {
        sinea[n] = 0;
        sinea[n+1] = 0;
        sinea[n+2] = 0;
        sinea[n+3] = 0;
        sinea[n+4] = 0;
        sinea[n+5] = 0;
        sinea[n+6] = 1000;
        sinea[n+7] = 1000;
        sinea[n+8] = 1000;
        sinea[n+9] = 1000;
        sinea[n+10] = 1000;
        sinea[n+11] = 1000;
        }

/*** Window Squarewave Signal ***/

/**        if(GenHamming( window, len) != EDSP_OK)  **/
/**        printf("problem with window");               **/

/**        if(VectorMult( sinea, sinea, window, len, res_shift) != EDSP_OK) **/
/**        printf("Problem with vectormult");                              **/


/*** Generate Sine Wave ***/

        if(sine == 0)
        {
        printf("Generating Sine Wave");
        k = len / 8;
        for(n = 0; n < len; n++)
        {
        sinea1[n] = floor(1000 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 32;
        for(n = 0; n < len; n++)
        {
        sineb[n] = floor(10 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");
        }

/*** Add Sine waves ***/

        for(n = 0; n < len; n++)
        {
        sinea[n] = sinea1[n] + sineb[n];
        }
/*** Create Imaginary Arrays ***/

        k = 0;
        for(n=0; n < NSAML; n=n+2)
        {
        imaga[n] = (2*sinea[k]);
        imaga[n+1] = 0;
        imagb[n] = (2*sinea[k]);
        imagb[n+1] = 0;
        k = k + 1;
        }


/*** Take FFTs Of Sine Waves ***/


        if(FftReal( fftouta, sinea, NSAM, scalefft) != EDSP_OK)
        printf("Problem With FFT");                                          


        if(FftReal( fftoutb, sineb, NSAM, scalefft) != EDSP_OK)
        printf("Problem With FFT");


        if(LogMagnitude( outputa, fftouta, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");


        if(LogMagnitude( outputb, fftoutb, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

/*** Complex FFT ***/

        if(FftComplex( fftoutc, imaga, NSAM, scalefft) != EDSP_OK)
        printf("Problem With FFT");
        
        if(FftComplex( fftoutd, imagb, NSAM, scalefft) != EDSP_OK)
        printf("Problem With FFT");

        if(LogMagnitude( outputai, fftoutc, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

        if(LogMagnitude( outputbi, fftoutd, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");


/*** Inverse FFT ***/

        if(IfftComplex( ifft, fftoutc, NSAM, scalefft) != EDSP_OK)
        printf("Problem With Ifft");


        FreeFft();



        
}

