/*
** FILE NAME:   FFT.C
**
** PROJECT ID:  DSPLIB SH  
**
**
** AUTHOR:      D. Maidment
**
** DESCRIPTION: This file Generates a sine wave, applies a Hamming window 
**              applies an FFT and then returns the solution in log format
**
** MODIFICATION HISTORY:
**
**              1.00    29-NOV-95
**
*/
#include <stdio.h>
#include <math.h>
#include <ensigdsp.h>
#include "testhead.h"


#define NSAM     128              /* number of samples */
#define NSAML    NSAM*2
#define NSAMS    NSAM/2
#define TWOPI     6.283185307
#define MAXSIZE   NSAML

/***************** Exercise To Take The FFT Of Data *******************/


void 
main()
{

        
        short   imaga[NSAML], imagb[NSAML], ifft[NSAML], sinea[NSAM], sineb[NSAM], fftouta[NSAM], fftoutb[NSAM], fftoutc[NSAML], fftoutd[NSAML], outputa[NSAMS], outputb[NSAMS], outputai[NSAMS], outputbi[NSAMS];
        int     n, k, len;
        float   scale;

        scale           =       1;
        len             =       NSAM;


/*** Initiate Twiddle Values ***/


        if(InitFft (MAXSIZE) != EDSP_OK)
        printf("Problem With Initialisation Of FFT");


/*** Generate Sine Wave ***/

        printf("Generating Sine Wave");
        k = len / 4;
        for(n = 0; n < len; n++)
        {
        sinea[n] = floor(200 * sin(TWOPI * k * n / len) + 0.5);
        }
        
        k = len / 16;
        for(n = 0; n < len; n++)
        {
        sineb[n] = floor(10 * sin(TWOPI * k * n / len) + 0.5);
        }

        printf("Done");

/*** Create Imaginary Arrays ***/

        k = 0;
        for(n=0; n < NSAML; n=n+2)
        {
        imaga[n] = (2*sinea[k]);
        imaga[n+1] = 0;
        imagb[n] = (2*sineb[k]);
        imagb[n+1] = 0;
        k = k + 1;
        }


/*** Take FFTs Of Sine Waves ***/


        if(FftReal(fftouta, sinea, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");                                          


        if(FftReal(fftoutb, sineb, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");


        if(LogMagnitude( outputa, fftouta, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");


        if(LogMagnitude( outputb, fftoutb, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

/*** Complex FFT ***/

        if(FftComplex(fftoutc, imaga, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");
        
        if(FftComplex(fftoutd, imagb, NSAM, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With FFT");

        if(LogMagnitude( outputai, fftoutc, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");

        if(LogMagnitude( outputbi, fftoutd, NSAMS, scale) != EDSP_OK)
        printf("problem With LogMagnitude");


/*** Inverse FFT ***/

        if(IfftReal( ifft, fftoutd, NSAML, EFFTALLSCALE) != EDSP_OK)
        printf("Problem With Ifft");


        FreeFft();



        
}

