/*
** FILE NAME:	fftcom.c
**
** PROJECT ID:	94-0040
**
** TITLE:	FftComplex
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION: This file contains the wrapper used to invoke a forward,
**		not-in-place, complex FFT.
**
**		int FftComplex(output, input, size, scale)
**
** PARAMETERS:
**		short output[]      : complex output data
**		const short input[] : complex input data
**		long size           : size of FFT
**		long scale          : scaling specification
**                                          No scaling = 0
**                                          halving on every stage = 0xffffffff
**
** RETURNS:	EDSP_OK      [0]  : success
**		EDSP_BAD_ARG [1]  : size < 2
**                                  size not a power of 2
**                                  size > max_fft_size
**
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>
extern int FftCore(short data[], long size, long scale);

int
FftComplex(short output[], const short input[], long size, long scale)
{
    long i;
    long test;

    /* Input Argument error testing */
    if ((size < 4) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* check that size is a power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    for (i = 0; i < size; i++)
    {
	output[2 * i] = input[2 * i];
	output[2 * i + 1] = input[2 * i + 1];
    }

    /*
     * The main FFT processing is performed in FftCore
     */
    return FftCore(output, size, scale);
}
