;
; FILE NAME:    fftinc.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        FftInComplex, IfftInComplex
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This function performs a forward, in-place, complex FFT.
;		This file merely provides the entry point. The warping of data
;		performed here allows the common FFT core to be performed
;		elsewhere.
;

; _FftInComplex and _IfftInComplex are the entry points.
;
; Three parameters are required. They are:
; 1.data address
; 2.size of data
; 3.scaling factor.
; These parameters are passed in registers R4, R5, R6.
; The return value of the function is
;       EDSP_OK      if the transform has been performed successfully,
;       EDSP_BAD_ARG if size<4, size>max_fft_size, or size not a power of 2.
;

	.EXPORT	_FftInComplex
	.EXPORT	_IfftInComplex
	.IMPORT	FFTICBV

; Forward/inverse is signalled to the core by a 0 or 1 at the bottom of the
; stack
forward	.EQU	0	;0=FFT,1=IFFT
inverse	.EQU	1

	.SECTION	P,CODE,ALIGN=4

_FftInComplex:
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-48,R15	;ALLOCATE PARAMETER AREA
	MOV	#forward,R1
	BRA	LB70
	MOV.L	R1,@R15

_IfftInComplex:
	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-48,R15	;ALLOCATE PARAMETER AREA
	MOV	#inverse,R1
	MOV.L	R1,@R15

LB70:	MOV.L	R4,@(4,R15)	;SAVE INADDR
	MOV.L	R4,@(12,R15)	;SAVE OUTADDR
	MOV.L	R5,@(8,R15)	;SAVE NO

; Now branch to the core
	MOV.L	BVADDR,R1
	JSR	@R1		;CALL FFT
	MOV.L	R6,@(20,R15)	;SAVE SCALE

EN:	ADD	#48,R15		;DEALLOCATE PAPAMETER AREA
	LDS.L	@R15+,PR	;POP REGISTERS FROM STACK
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV.L	@R15+,R8
	MOV.L	@R15+,R9
	MOV.L	@R15+,R10
	MOV.L	@R15+,R11
	MOV.L	@R15+,R12
	MOV.L	@R15+,R13
	RTS			;RETURN TO CALLING C PROGRAM
	MOV.L	@R15+,R14

	.ALIGN	4
BVADDR	.data.l	FFTICBV

	.END
