/*
** FILE NAME:	fftincom.c
**
** PROJECT ID:	94-0040
**
** TITLE:	FftInComplex
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This file contains the wrapper used to invoke a forward,
**		in-place, complex FFT.
**
**		int FftInComplex(data, size, scale)
**
** PARAMETERS:
**              short data[]  : Complex  data
**              long size     : size of FFT
**              long scale    : scaling specification
**                            		No scaling = 0
**                                      Complete scaling = 0xffffffff
**
** RETURNS :
**              EDSP_OK       : success
**              EDSP_BAD_ARG  : size < 2
**         			size not a power of 2.
**                              size > max_fft_size
**
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>
extern int FftCore(short data[], long size, long scale);

int 
FftInComplex(short data[], long size, long scale)
{
    long test;

    /* Input Argument error testing */
    if ((size < 4) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* test for power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    /*
     * The main FFT processing is performed in FftCore
     */
    return FftCore(data, size, scale);
}
