/*
** FILE NAME:	gengwn.c
**
** PROJECT ID:	94-0040
**
** TITLE:	GenGWnoise
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine generates Gaussian white noise. Samples are produced
**              in pairs using the modified Box-muller method.
**
**		int GenGWnoise(output, no_samples, variance)
**
** PARAMETERS:
**              short output[]      : output noise samples.
**              long no_samples     : number of samples required.
**              float variance      : variance of noise distribution.
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : no_elements < 1
**                                    variance <= 0.0
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <stdlib.h>
#include <math.h>
#include <ensigdsp.h>

int
GenGWnoise(short output[], long no_samples, float variance)
{
    /* Variable Declaration */
    int i;
    double r1, r2, x;

    /* Input Argument error checking */
    if ((no_samples < 1) || (variance <= 0.0))
	return EDSP_BAD_ARG;

    for (i = 0; i < no_samples; i += 2)
    {
	do
	{
	    r1 = (rand() - 16383.5) / 16383.5;	/* Get r1 between -1 and 1 */
	    r2 = (rand() - 16383.5) / 16383.5;	/* Get r2 between -1 and 1 */
	    x = r1 * r1 + r2 * r2;
	}
	while ((r1 > 1.0) || (r1 < -1.0) ||
	       (r2 > 1.0) || (r2 < -1.0) || (x >= 1.0));

	output[i] = floor(sqrt(variance) * r1 * sqrt(-2 * log(x) / x) + 0.5);

	if ((i + 1) != no_samples)
	    output[i + 1] = floor(sqrt(variance) * r2 *
				  sqrt(-2 * log(x) / x) + 0.5);
    }

    return EDSP_OK;		/* Return Success */
}
