/*
** FILE NAME:	hamming.c
**
** PROJECT ID:	94-0040
**
** TITLE:	GenHamming
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This file contains the routines used to generate the Hamming
**              window.
**
**		int GenHamming(output, win_size)
**
** PARAMETERS:
**              short output[]       : array to contain window
**              long win_size        : size of window
**
** RETURNS:     EDSP_OK              : success
**              EDSP_BAD_ARG         : win_size < 2
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>
#define TWOPI 6.28318530717959

int
GenHamming(short output[], long win_size)
{
    /* variable Declaration */
    int i;
    double pi2;
    double temp;

    if (win_size <= 1)		/* Input Argument error checking */
	return EDSP_BAD_ARG;

    pi2 = TWOPI / win_size;	/* 2*pi / win_size */

    /* Check for even number */
    if ((win_size % 2) == 0)
    {
	output[0] = 2621;
	output[win_size / 2] = 32767;
	for (i = 1; i <= win_size / 4; i++)	/* loop for even number */
	{
	    temp = cos(pi2 * i);

	    output[win_size / 2 + i] = floor((32767 * (0.54 + 0.46 * temp) + 0.5));
	    output[win_size / 2 - i] = output[win_size / 2 + i];

	    output[win_size - i] = floor((32767 * (0.54 - 0.46 * temp) + 0.5));
	    output[i] = output[win_size - i];

	}
    }
    else
    {
	output[0] = 2621;
	for (i = 1; i <= win_size / 2; i++)	/* Loop for odd number */
	{
	    output[i] = floor((32767 * (0.54 - 0.46 * cos(pi2 * i)) + 0.5));
	    output[win_size - i] = output[i];
	}
    }

    return EDSP_OK;		/* Return success */

}
