/*
** FILE NAME:   Ifftirel.c
**
** PROJECT ID:  94-0040
**
** TITLE:       IfftInReal
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This file contains the wrapper used to invoke an inverse,
**              in-place, real FFT.
**
**              IfftInReal(data, size, scale)
**
** PARAMETERS:
**              short data[]        : Complex  data
**              long size           : size of FFT
**              long scale          : scaling specification
**                                      No scaling = 0
**                                      Complete scaling = 0xffffffff
**
** RETURNS:     EDSP_OK       : success
**              EDSP_BAD_ARG  : size < 2
**                              size not a power of 2.
**                              size > max_fft_size
**
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>

int 
IfftInReal(short data[], long size, long scale)
{
    /* Variable declaration */
    int i, index;
    long h1r, h1i, h2r, h2i;
    int test;

    /* Input Argument error testing */
    if ((size < 8) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* test for power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    index = max_fft_size / size;/* Twiddle factor index scaling */

    /* Main Loop */
    for (i = 1; i <= (size >> 2); i++)
    {
	h1r = (long)data[2 * i] + data[size - 2 * i];
	h1i = (long)data[2 * i + 1] - data[size - 2 * i + 1];
	h2r = -((long)data[2 * i + 1] + data[size - 2 * i + 1]);
	h2i = (long)data[2 * i] - data[size - 2 * i];

	data[2 * i] = ((h1r << 15) + twtble[index * 2 * i] * h2r - twtble[index * 2 * i + 1] * h2i + 32768) >> 16;
	data[2 * i + 1] = ((h1i << 15) + twtble[index * 2 * i] * h2i + twtble[index * 2 * i + 1] * h2r + 32768) >> 16;
	data[(size - 2 * i)] = ((h1r << 15) + twtble[index * 2 * i + 1] * h2i - twtble[index * 2 * i] * h2r + 32768) >> 16;
	data[(size - 2 * i + 1)] = (-(h1i << 15) + twtble[index * 2 * i] * h2i + twtble[index * 2 * i + 1] * h2r + 32768) >> 16;
    }

    data[0] = ((h1r = data[0]) + data[1] + 1) >> 1;     /* Rearrange the first
							 * and last real
							 * elements */
    data[1] = (h1r - data[1] + 1) >> 1;

    /* Inverse FFT procedure call */
    if ((i = IfftInComplex(data, (size >> 1), scale)) != EDSP_OK)
	return EDSP_BAD_ARG;

    return EDSP_OK;
}
