/*
** FILE NAME:   Ifftreal.c
**
** PROJECT ID:  94-0040
**
** TITLE:       IfftReal
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: This file contains the wrapper used to invoke an inverse,
**              not-in-place, real FFT.
**
**              int IfftReal(output,input,  size, scale)
**
** PARAMETERS:
**              short output[]      : Complex output  data
**              const short input[] : complex input data
**              long size           : size of FFT
**              long scale          : scaling specification
**                                      No scaling = 0
**                                      Complete scaling = 0xffffffff
**
** RETURNS :
**              EDSP_OK       : success
**              EDSP_BAD_ARG  : size < 2
**                              size not a power of 2.
**                              size > max_fft_size
**
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>

int 
IfftReal(short output[], const short input[], long size, long scale)
{
    /* Variable declaration */
    int i, index;
    long h1r, h1i, h2r, h2i;
    int test;

    /* Input Argument error testing */
    if ((size < 8) || (size > max_fft_size))
	return EDSP_BAD_ARG;

    /* test for power of 2 */
    for (test = size; test > 2; test /= 2)
	if (test & 1)
	    return EDSP_BAD_ARG;

    for (i = 0; i < (size >> 1); i++)
    {
	output[2 * i] = input[2 * i];
	output[2 * i + 1] = input[2 * i + 1];
    }

    index = max_fft_size / size;/* Twiddle factor index scaling */

    /* Main Loop */
    for (i = 1; i <= (size >> 2); i++)
    {
	h1r = ((long)output[2 * i] + output[size - 2 * i]);
	h1i = ((long)output[2 * i + 1] - output[size - 2 * i + 1]);
	h2r = -((long)output[2 * i + 1] + output[size - 2 * i + 1]);
	h2i = ((long)output[2 * i] - output[size - 2 * i]);

	output[2 * i] = ((h1r << 15) + twtble[index * 2 * i] * h2r - twtble[index * 2 * i + 1] * h2i + 32768) >> 16;
	output[2 * i + 1] = ((h1i << 15) + twtble[index * 2 * i] * h2i + twtble[index * 2 * i + 1] * h2r + 32768) >> 16;
	output[(size - 2 * i)] = ((h1r << 15) + twtble[index * 2 * i + 1] * h2i - twtble[index * 2 * i] * h2r + 32768) >> 16;
	output[(size - 2 * i + 1)] = (-(h1i << 15) + twtble[index * 2 * i] * h2i + twtble[index * 2 * i + 1] * h2r + 32768) >> 16;
    }

    output[0] = ((h1r = output[0]) + output[1] + 1) >> 1;       /* Rearrange the first
								 * and last real
								 * elements */
    output[1] = (h1r - output[1] + 1) >> 1;

    /* Inverse FFT procedure call */
    if ((i = IfftInComplex(output, (size >> 1), scale)) != EDSP_OK)
	return EDSP_BAD_ARG;

    return EDSP_OK;
}
