/*
** FILE NAME:	initiir.c
**
** PROJECT ID:	94-0040
**
** TITLE:	InitIir
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	Routine to allocate and initialise
**              workspace memory for IIR filtering
**
**		int InitIir(workspace, no_sections)
** PARAMETERS:
**              short **workspace   : double pointer to filter memory
**              long no_sections    : number of 2nd order filter sections K
**
** RETURNS:     EDSP_OK             : success
**              EDSP_BAD_ARG        : no_sections < 1
**              EDSP_NO_HEAP        : insufficient space available from
**                                    malloc.
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <stdio.h>
#include <stdlib.h>
#include <ensigdsp.h>

int 
InitIir(short **workspace, long no_sections)
{
    /* Variable declaration */
    int i;

    /* Input Argument Error checking */
    if (no_sections < 1)
	return EDSP_BAD_ARG;

    /* memory allocation */
    if ((*workspace = (short *)malloc((2 * no_sections) * sizeof(short))) == NULL)	/* storage space
											 * checking */
	 return EDSP_NO_HEAP;

    /* workspace initialisation */
    for (i = 0; i < (2 * no_sections); i++)
	(*workspace)[i] = 0;

    return EDSP_OK;		/* Return Success */
}
