;
; FILE NAME:    lms1.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        Lms1
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the single sample adaptive FIR filter.
;
;               int Lms1(output, input, coeff, no_coeffs, res_shift, conv_fact,
;                        workspace)
;
; PARAMETERS:   short   *output
;         	short   input
;               long    no_samples
;        	short   coeff[]
;               long    no_coeffs
;               int     res_shift
;               short   *workspace
;

	.EXPORT	_Lms1
	.IMPORT	SHT

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack and load stack parameters
_Lms1:	MOV.L	@(0,R15),R2	;R9=N_COEF
	MOV.L	@(4,R15),R0	;R0=res_shift
	MOV.L	@(8,R15),R1	;R14=2U
	MOV.L	@(12,R15),R3	;R3=WORKSPACE
	MOV	R14,@-R15
	MOV	R1,R14		;R14=2U
	MOV	R9,@-R15
	MOV	R2,R9		;R9=N_COEF
	MOV	R8,@-R15
	MOV	R13,@-R15
	MOV	R12,@-R15
	MOV	R11,@-R15
	MOV	R10,@-R15
	MOV	R9,R10		;R9=R10=N_COEF
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15
	ADD	#-16,R15
	MOV.L	R6,@(4,R15)	;STORE REF SIGNAL POINTER
	MOV	R7,R8		;R7=R8=COEF
	MOV.L	R14,@(8,R15)	;STORE 2U
	MOV	R9,R14
	MOV.L	R0,@(12,R15)	;STORE res_shift

	SHLL	R14		;N_COEF*2
	ADD	R3,R14		;R14 -> LAST-2
	ADD	#4,R14		;R14 -> LAST+2

;ERROR CHECKING
	CMP/PZ	R0		;res_shift >=0 ?
	BF	ERR
	MOV	#27,R1
	CMP/GT	R1,R0		;res_shift > 27 ?
	BT	ERR
	MOV	#2,R1
	CMP/GT	R1,R9		;N_COEF > 2 ?
	BT	argsOK

ERR:	BRA	RTN
	MOV	#1,R0		;R0=RETURN CODE

	.ALIGN	4
argsOK:	MOV.W	NONSAT,R2
	STC	SR,R1
	AND	R2,R1		;SET S BIT TO 0
	LDC	R1,SR

	SHLL2	R0
	SHLL2	R0
	MOV	SHTADDR,R2	;R2=ADDRESS OF SHIFT ROUTINE
	SHLL	R0
	MOV.L	@R3+,R1		;READ CURRENT POSITION IN CIRCULAR DELAY LINE
	ADD	R2,R0		;R0=SHIFT ROUTINE ADDRESS

; A circular delay buffer is implemented as two halves.
; First check whether we are already at the end of the array
newsample:
	CMP/EQ	R1,R3		;CURRENT POS = FIRST ?
	BF	LB50		;NO
	MOV	R14,R1		;YES, RESET TO LAST+2
LB50:	MOV	R7,R8		;R8=R7 -> COEF
	MOV.W	R5,@-R1		;I/P => CURRENT POS
	MOV	R14,R13		;R13=R14=LAST+2
	MOV	R1,R2		;R2=R1 -> CURRENT POS

;				;N_COEF=N_LOW+N_UP
;				;N_UP=LAST POS-CURRENT POS+1
	SUB	R1,R13		;GET N_UP
	SHLR	R13		;/2

; Calculate dot product of the first half of the circular buffer
	CLRMAC			;CLEAR MAC
	MOV.L	R13,@(0,R15)	;STORE N_UP
	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	MOV	LOWMASK,R12	;MASK AWAY LOWER 2 BITS
	AND	R13,R11		;R11=LOWER 2 BITS
	AND	R13,R12		;R12=UPPER 30 BITS

LB1:	SHLL	R11		;R11*2
	MOV	half1ad,R13	;R13=ADDRESS OF half1
	SUB	R11,R13		;R13=ADDRESS OF half1 - R11*2
	JMP	@R13		;R13=JUMP ADDRESS
	NOP

NONSAT:	.data.w	H'0FFFD

	.ALIGN	4
	MAC	@R2+,@R8+	;MULTIPLY DATA IN WORKSPACE AND COEF
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+

half1:	CMP/PL	R12
	BF	doneloop1
loop1:	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	ADD	#-4,R12
	MAC	@R2+,@R8+
	CMP/PL	R12
	MAC	@R2+,@R8+
	BT	loop1


doneloop1:
	MOV.L	@(0,R15),R11	;POP N_UP
	MOV	R9,R2		;R2=R2=N_COEF
	SUB	R11,R2		;N_LOW=N_COEF-N_UP
	MOV	UPMASK,R11	;MASK UPPER 30 BITS
	MOV	LOWMASK,R12	;MASK LOWER 2 BITS
	AND	R2,R11		;R11= LOWER 2 BITS
	AND	R2,R12		;R12= UPPER 30 BITS
	MOV	R3,R2		;R2=LAST, RESET TO FIRST

; Do the second half of the dot product
LB10:	MOV	half2ad,R13	;R13= ADDRESS OF half2
	SHLL	R11
	SUB	R11,R13
	JMP	@R13		;R13=JUMP ADDRESS
	NOP

	.ALIGN	4
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	MAC	@R2+,@R8+

half2:	CMP/PL	R12
	BF	doneloop2
loop2:	MAC	@R2+,@R8+
	MAC	@R2+,@R8+
	ADD	#-4,R12
	MAC	@R2+,@R8+
	CMP/PL	R12
	MAC	@R2+,@R8+
	BT	loop2

; Call the shift routine
doneloop2:
	STS	MACH,R13	;MSB'S IN R13
	JSR	@R0
	STS	MACL,R8		;LSB'S IN R8

END1:	EXTS.W	R8,R8
	MOV.W	R8,@R4		;STORE OUTPUT
	ADD	#2,R4


; Update filter coefficients
ADP:	MOV.L	@(4,R15),R11	;READ D
	MOV	R9,R10		;R10=N_COEF

	SUB	R8,R11		;R11=D-OUTPUT=ERR
	EXTS.W	R11,R8
	CMP/EQ	R8,R11		;OVERFLOW ?
	BT	ADP1
	CMP/PZ	R11		; T=1 if +ve, 0 if -ve
	MOV.W	SATVAL,R8
	NEGC	R8,R11
	EXTS.W	R11,R11		;SATURATE R11

ADP1:	MOV.L	@(8,R15),R2	;READ 2u
	MULS	R2,R11		;2U*ERR
	MOV.W	ROUND,R13	;ADD ROUNDING
	MOV	R7,R8		;R8=R7 -> COEF
	STS	MACL,R2
	ADD	R13,R2
	SHAL	R2
	SHLR16	R2		;>>15

loop3:	MOV.W	@R1+,R11	;READ PRESENT INPUT FROM DELAY LINE
	CMP/GE	R14,R1		;DELAY LINE POINTER R1 >= LAST +2 ?
	BF	ADP3
	MOV	R3,R1		;RESET R1 TO FIRST
ADP3:	MULS	R2,R11		;(2U*ERR)*X(N)
	STS	MACL,R12
	ADD	R13,R12		;ADD ROUNDING
	SHAL	R12		;<<1
	MOV.W	@R8,R11		;READ COEF
	SHLL16	R11		;<< 16
	ADDV	R11,R12		;COEF+(2U*ERR)*X(N)
	BF	ADP4		;OVERFLOW ?
	CMP/PZ	R12
	MOV.L	MAX,R12		;POSITIVE SATURATION
	BF	ADP4
	MOV.L	MIN,R12		;NEGATIVE SATURATION

ADP4:	SHLR16	R12		;RESULT >> 16
	MOV.W	R12,@R8		;UPDATE COEF
	ADD	#2,R8		;INCREASE COEF POINTER
	ADD	#-1,R10		;DECREASE LOOP COUNTER
	CMP/PL	R10
	BT	loop3

	MOV.L	R1,@-R3		;SAVE CURRENT POSITION
	MOV	#0,R0		;RETURN CODE

RTN:	ADD	#16,R15
	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	MOV	@R15+,R8
	MOV	@R15+,R9
	RTS
	MOV	@R15+,R14


	.ALIGN	4
half1ad	.data.l	half1
half2ad	.data.l	half2
SHTADDR	.data.l	SHT
UPMASK	.data.l	H'00000003
LOWMASK	.data.l	H'FFFFFFFC
MAX	.data.l	H'7FFFFFFF
MIN	.data.l	H'80000000
ROUND	.data.w	H'4000
SATVAL	.data.w H'08000


	.END

