/*
**
** FILE NAME:	logmag.c
**
** PROJECT ID:	94-0040
**
** TITLE:	Log Magnitude
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine calculates the log magnitude of the complex
**              input data in decibels and writes the scaled result into
**              the output array.
**
** 		int LogMagnitude(short output[], const short input[],
**                               long no_elements, float fscale)
**
** PARAMETERS:
**              short output[]      : Real output data
**              const short input[] : Complex input data
**              long no_elements    : number of log Magnitude outputs required
**              float fscale        : output scaling factor
**
** RETURNS:	EDSP_OK       : success
**              EDSP_BAD_ARG  : no_elements < 1
**				no_elements > 32767
**                              |fscale| >= 2^15/ (10*log10 2^31)
**
** MODIFICATION HISTORY:
**              1.0  20-7-95    Initial revision
*/

#include <ensigdsp.h>
#include <math.h>

int
LogMagnitude(short output[], const short input[], long no_elements,
	     float fscale)
{
    long count = 0;

    /* Input Argument Error checking */
    if ((no_elements < 1) || (no_elements > 32767) || (fscale >= 351.1385155))
	return EDSP_BAD_ARG;

    while (no_elements > count)
    {
	output[count] = (short)floor(10.0 * fscale *
			  (log10(((long)input[2 * count] * input[2 * count]) +
		       ((long)input[2 * count + 1] * input[2 * count + 1]))) +
				     0.5);

	count++;
    }

    return EDSP_OK;
}
