/*
** FILE NAME:	mean.c
**
** PROJECT ID:	94-0040
**
** TITLE:	Mean
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine calculates the mean of the input data
**
**		int Mean(mean, input, no_elements)
**
** PARAMETERS:
**              short *mean            : mean of the data
**              const short input[]    : array of input data
**              long  no_elements      : size of the input buffer
**
** RETURNS:     EDSP_OK                : success
**              EDSP_BAD_ARG           : no_elements < 1
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int 
Mean(short *mean, const short input[], long no_elements)
{
    /* Variable Declaration */
    int i;
    double temp;

    /* Input argument error checking */
    if (no_elements < 1)
	return EDSP_BAD_ARG;

    temp = 0;			/* sum initialisation */

    for (i = 0; i < no_elements; i++)	/* Main Loop */
	temp += input[i];

    /* rounding */
    if (temp < 0)
	temp = ceil(temp / no_elements - 0.5);
    else
	temp = floor(temp / no_elements + 0.5);

    if (temp < -32768)
	*mean = -32768;
    else if (temp > 32767)
	*mean = 32767;
    else
	*mean = temp;

    return EDSP_OK;		/* Return success */
}
