;
; FILE NAME:    shift.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        sht*
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the shift subroutines used by many of the
;		functions in DSPLib-SH. Each subroutine provides a right shift
;		of between 0 and 27 bits, followed by rounding of the discarded
;		least significant bits and saturation if overflow of the output
;		16 bits would occur.
;
;		The 42 bit input is supplied in R8 and R13.
;		The 16 bit result is returned in R8.
;		The T flag should be clear when these functions are called.
;		R2, R11, R12 are used as scratch registers.
;

	.EXPORT	SHT

; ENTRY POINTS are calculated as (res_shift*16 words) + SHT
;
	.SECTION	P,CODE,ALIGN=4
SHT:
;ROUTINE FOR 0 SHIFT
SHT0:	MOV	R8,R12
	XTRCT	R13,R12		;R12=SIGN + 26 MSB
	SWAP.W	R12,R11
	EXTS.W	R11,R11		;R11=SIGN
	CMP/EQ	R12,R11		;IS R12 SIGN ONLY ?
	BF	SAT3		;SATURATE IF NOT
	EXTS.W	R8,R2
	CMP/EQ	R8,R2		;IS THE UPPER 17 BITS OF R8 SIGN ONLY ?
	BF	SAT3		;SATURATE IF NOT
	RTS
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

;ROUTINE FOR 1 SHIFT
SHT1:	MOV.W	ROUND1,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK1,R2
	TST	R2,R11
	BF	SAT
	RTS
	SHLR	R8
	NOP
	NOP
MASK1	.data.w	H'0FFFE
ROUND1	.data.w	H'0001


SHT2:	MOV.W	ROUND2,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK2,R2
	TST	R2,R11
	BF	SAT
	RTS
	SHLR2	R8

SAT3:	BRA	SAT1
	NOP
MASK2	.data.w	H'0FFFC
ROUND2	.data.w	H'0002


SHT3:	MOV.W	ROUND3,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK3,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	RTS
	SHLR	R8
	NOP
MASK3	.data.w	H'0FFF8
ROUND3	.data.w	H'0004

SHT4:	MOV.W	ROUND4,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK4,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	RTS
	SHLR2	R8
	NOP
MASK4	.data.w	H'0FFF0
ROUND4	.data.w	H'0008

SHT5:	MOV.W	ROUND5,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK5,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	SHLR2	R8
	RTS
	SHLR	R8
MASK5	.data.w	H'0FFE0
ROUND5	.data.w	H'0010

SHT6:	MOV.W	ROUND6,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK6,R2
	TST	R2,R11
	BF	SAT
	SHLL2	R8
	RTS
	SHLR8	R8
	NOP
MASK6	.data.w	H'0FFC0
ROUND6	.data.w	H'0020

SHT7:	MOV.W	ROUND7,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK7,R2
	TST	R2,R11
	BF	SAT
	SHLL	R8
	RTS
	SHLR8	R8
	NOP
MASK7	.data.w	H'0FF80
ROUND7	.data.w	H'0040


SHT8:	MOV.W	ROUND8,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK8,R2
	TST	R2,R11
	BF	SAT
	RTS
	SHLR8	R8

SAT:	BRA	SAT1
	NOP
MASK8	.data.w	H'0FF00
ROUND8	.data.w	H'0080


SHT9:	MOV.W	ROUND9,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK9,R2
	TST	R2,R11
	BF	SAT
	SHLR	R8
	RTS
	SHLR8	R8
	NOP
MASK9		.data.w	H'0FE00
ROUND9	.data.w	H'0100

SHT10:	MOV.W	ROUND10,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK10,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	RTS
	SHLR8	R8
	NOP
MASK10		.data.w	H'0FC00
ROUND10	.data.w	H'0200


SHT11:	MOV.W	ROUND11,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK11,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	SHLR8	R8
	RTS
	SHLR	R8
MASK11		.data.w	H'0F800
ROUND11	.data.w	H'0400

SHT12:	MOV.W	ROUND12,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK12,R2
	TST	R2,R11
	BF	SAT
	SHLR2	R8
	SHLR8	R8
	RTS
	SHLR2	R8
MASK12		.data.w	H'0F000
ROUND12	.data.w	H'0800


SHT13:	MOV.W	ROUND13,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK13,R2
	TST	R2,R11
	BF	SAT
	SHLL2	R8
	SHLL	R8
	RTS
	SHLR16	R8
MASK13		.data.w	H'0E000
ROUND13	.data.w	H'1000

SHT14:	MOV.W	ROUND14,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK14,R2
	TST	R2,R11
	BF	SAT
	SHLL2	R8
	RTS
	SHLR16	R8
	NOP
MASK14		.data.w	H'0C000
ROUND14	.data.w	H'2000


SHT15:	MOV.W	ROUND15,R2
	ADDC	R2,R8
	MOV	#0,R11
	ADDC	R11,R13
	MOV	R8,R11
	XTRCT	R13,R11
	ADD	R2,R11
	MOV.W	MASK15,R2
	TST	R2,R11
	BF	SAT
	SHLL	R8
	RTS
	SHLR16	R8
	NOP
MASK15		.data.w	H'8000
ROUND15	.data.w	H'4000


SHT16:	MOV.W	ROUND16,R2
	CLRT
	EXTU.W	R2,R2
	ADDC	R2,R8
	MOV	#0,R2
	ADDC	R2,R13
;	BT	SAT
	XTRCT	R13,R8		;EXTRACT 16 LSB OF R13 AND 16 MSB OF R8
	EXTS.W	R8,R2		;R2=SIGN EXTENDED R8
	CMP/EQ	R8,R2
	BF	SAT		;SATURATE IF R2 != R8
	RTS
	NOP
	NOP
	NOP
ROUND16	.data.w	H'8000
	NOP

SHT17:	MOV.W	ROUND17,R2
	XTRCT	R13,R8		;get R13 16 lsb and R8 16 msb
	ADD	R2,R8		;ADD ROUNDING
	SHAR	R8		;right shift as required
	EXTS.W	R8,R2		;R2=SIGN EXTENDED R8
	CMP/EQ	R8,R2
	BF	SAT2		;SATURATE IF R2 != R8
	RTS
	NOP
	NOP
	NOP
	NOP
	NOP
SATCHK17	.data.w	H'0002
MASK17		.data.w	H'0FFFC
ROUND17		.data.w	H'0001



SHT18:	MOV.W	ROUND18,R2
	XTRCT	R13,R8		;get R13 16 lsb and R8 16 msb
	ADD	R2,R8		;ADD ROUNDING
	SHAR	R8		;right shift as required
	SHAR	R8
	EXTS.W	R8,R2		;R2=SIGN EXTENDED R8
	CMP/EQ	R8,R2
	BF	SAT2		;SATURATE IF R2 != R8
	RTS
	NOP
	NOP
SATCHK18	.data.w	H'0004
MASK18		.data.w	H'0FFF8
ROUND18	.data.w	H'0002
	NOP
	NOP

SHT19:	MOV.W	ROUND19,R2
	XTRCT	R13,R8		;get R13 16 lsb and R8 16 msb
	ADD	R2,R8		;ADD ROUNDING
	SHAR	R8		;right shift as required
	SHAR	R8
	SHAR	R8
	EXTS.W	R8,R2		;R2=SIGN EXTENDED R8
	CMP/EQ	R8,R2
	BF	SAT2		;SATURATE IF R2 != R8
	RTS
	NOP
	NOP
	NOP
SATCHK19	.data.w	H'0008
MASK19		.data.w	H'0FFF0
ROUND19	.data.w	H'0004

SHT20:	MOV.W	ROUND20,R2
	XTRCT	R13,R8
	MOV.L	SATCHK20,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK20,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT2
	SHLR2	R8
	RTS
	SHLR2	R8
ROUND20	.data.w	H'0008
SATCHK20	.data.l	H'00080000
MASK20		.data.l	H'0FFF00000

SHT21:	MOV.W	ROUND21,R2
	XTRCT	R13,R8		;get R13 16 lsb and R8 16 msb
	ADD	R2,R8		;ADD ROUNDING
	SHAR	R8		;right shift as required
	SHAR	R8
	SHAR	R8
	SHAR	R8
	SHAR	R8
	EXTS.W	R8,R2		;R2=SIGN EXTENDED R8
	CMP/EQ	R8,R2
	BF	SAT1		;SATURATE IF R2 != R8
	RTS
	NOP
SATCHK21	.data.w	H'0010
MASK21		.data.w	H'0FFE0
ROUND21	.data.w	H'0010



SHT22:	MOV.W	ROUND22,R2
	XTRCT	R13,R8
	MOV.L	SATCHK22,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK22,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	SHLL2	R8
	RTS
	SHLR8	R8
ROUND22	.data.w	H'0020
SATCHK22	.data.l	H'00200000
MASK22		.data.l	H'0FFC00000


SHT23:	MOV.W	ROUND23,R2
	XTRCT	R13,R8
	MOV.L	SATCHK23,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK23,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	SHLL	R8
	RTS
	SHLR8	R8
ROUND23	.data.w	H'0040
SATCHK23	.data.l	H'00400000
MASK23		.data.l	H'0FF800000


SHT24:	MOV.W	ROUND24,R2
	XTRCT	R13,R8
	MOV.L	SATCHK24,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK24,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	RTS
	SHLR8	R8
ROUND24	.data.w	H'0080
	NOP
SAT2:	BRA	SAT1
	NOP
	NOP
	NOP

SHT25:	MOV.W	ROUND25,R2
	XTRCT	R13,R8
	MOV.L	SATCHK25,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK25,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	SHLR	R8
	RTS
	SHLR8	R8
ROUND25	.data.w	H'0100
	NOP
	NOP
	NOP
	NOP

SHT26:	MOV.W	ROUND26,R2
	XTRCT	R13,R8
	MOV.L	SATCHK26,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK26,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	SHLR2	R8
	RTS
	SHLR8	R8
ROUND26	.data.w	H'0200
	NOP
	NOP
	NOP
	NOP


SHT27:	MOV.W	ROUND27,R2
	XTRCT	R13,R8
	MOV.L	SATCHK27,R11
	ADD	R2,R8		;ADD ROUNDING
	MOV.L	MASK27,R2
	ADD	R8,R11
	TST	R2,R11
	BF	SAT1
	SHLR2	R8
	SHLR	R8
	RTS
	SHLR8	R8
ROUND27	.data.w	H'0400
	NOP
	NOP
	NOP


	.ALIGN	4
SATCHK24	.data.l	H'00800000
MASK24		.data.l	H'0FF000000
SATCHK25	.data.l	H'01000000
MASK25		.data.l	H'0FE000000
SATCHK26	.data.l	H'02000000
MASK26		.data.l	H'0FC000000
SATCHK27	.data.l	H'04000000
MASK27		.data.l	H'0F8000000


SAT1:	MOV.W	SATVAL,R8
	CMP/PZ	R13	; T=1 if +ve, 0 if -ve
	RTS
	NEGC	R8,R8

SATVAL	.data.w H'08000


	.END
