/*
** FILE NAME:   tcorr.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB Correlate() function
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Tcorr() checks that Correlate():
**              a) Does not corrupt the stack
**              b) Passes a simple test vector
**              c) Correctly flags incorrect arguments
**              d) Passes a bit exact test
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "tcorr.txt"

#define NX      5
#define NW      5
#define M       7
#define MAXM    10
#define MAXSH   27
#define NARGS   7

void 
Tcorr(char *rcode)
{
    short i, j, n, errc;
    short output[MAXM];
    int xsize, wsize, ncorr, rshift;
    long sum;

    static int stkchk[NARGS];



/* test working of stack */
    xsize = NX;
    wsize = NW;
    ncorr = M;
    rshift = 15;
    stkchk[0] = (int)output;
    stkchk[1] = (int)w;
    stkchk[2] = (int)x;
    stkchk[3] = wsize;
    stkchk[4] = xsize;
    stkchk[5] = ncorr;
    stkchk[6] = rshift;

    /* also test for OK return argument */
    if (Correlate(output, w, x, wsize, xsize, ncorr, rshift) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)w ||
	stkchk[2] != (int)x ||
	stkchk[3] != wsize ||
	stkchk[4] != xsize ||
	stkchk[5] != ncorr ||
	stkchk[6] != rshift)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 */
    errc = 0;
    for (i = 0; i < M; i++)
	if (output[i] != t1[i])
	{
	    errc++;
	}
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    for (xsize = -2; xsize < 1; xsize++)
	if (Correlate(output, w, x, NW, xsize, M, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (wsize = -2; wsize < 1; wsize++)
	if (Correlate(output, w, x, wsize, NX, M, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    if (Correlate(output, w, x, NW, NW - 1, M, 0) != EDSP_BAD_ARG)
	sprintf(rcode, "Bad Return Argument\n");
    for (ncorr = -2; ncorr < 1; ncorr++)
	if (Correlate(output, w, x, NW, NX, ncorr, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (Correlate(output, w, x, NW, NX, M, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = 28; rshift < 45; rshift++)
	if (Correlate(output, w, x, NW, NX, M, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    for (i = 0; i < MAXM; output[i++] = 0) ;
    j = 0;
    errc = 0;
    for (wsize = 1; wsize <= NW; wsize++)
	for (xsize = wsize; xsize <= NX; xsize++)
	    for (rshift = 0; rshift <= MAXSH; rshift++)
	    {
		ncorr = wsize;
		if (Correlate(output, w, x, wsize, xsize, ncorr, rshift) != EDSP_OK)
		    sprintf(rcode, "EDSP_OK not returned\n");
		sum = 0;
		for (n = 0; n < ncorr; n++)
		    sum += output[n];
		if (sum != sumcorr[j])
		{
		    errc++;
		}
		j++;
	    }
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Correlate Passed\n");
    else
	strcat(rcode, "Correlate Failed\n");

}
