/*
** FILE NAME:	tcorrcyc.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB CorrCyclic() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tcorrcyc() checks that CorrCyclic():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Correctly flags incorrect arguments
**		d) Passes a bit exact test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	N	5
#define MAXSH	27
#define	NARGS	5

void 
Tcorrcyc(char *rcode)
{
    short i, j, n, errc;
    short output[N];
    int size, rshift;
    long sum;

    static int stkchk[NARGS];

#include "tcorrcyc.txt"

/* test working of stack */
    size = N;
    rshift = 15;
    stkchk[0] = (int)output;
    stkchk[1] = (int)w;
    stkchk[2] = (int)x;
    stkchk[3] = size;
    stkchk[4] = rshift;

    /* also test for OK return argument */
    if (CorrCyclic(output, w, x, size, rshift) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)w ||
	stkchk[2] != (int)x ||
	stkchk[3] != size ||
	stkchk[4] != rshift)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 */
    errc = 0;
    for (i = 0; i < N; i++)
	if (output[i] != t1[i])
	{
	    errc++;
	}
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    for (size = -2; size < 1; size++)
	if (CorrCyclic(output, w, x, size, 0) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (CorrCyclic(output, w, x, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = 28; rshift < 45; rshift++)
	if (CorrCyclic(output, w, x, N, rshift) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    for (i = 0; i < N; output[i++] = 0) ;
    j = 0;
    errc = 0;
    for (size = 1; size <= N; size++)
	for (rshift = 0; rshift <= MAXSH; rshift++)
	{
	    if (CorrCyclic(output, w, x, size, rshift) != EDSP_OK)
		sprintf(rcode, "EDSP_OK not returned\n");
	    sum = 0;
	    for (n = 0; n < size; n++)
		sum += output[n];
	    if (sum != sumcorrcyc[j])
	    {
		errc++;
	    }
	    j++;
	}
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "CorrCyclic Passed\n");
    else
	strcat(rcode, "CorrCyclic Failed\n");
}
