/*
** FILE NAME:	tstdiir1.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB DIir1() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tstdiir1() checks that DIir1():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Handles small array sizes
**		d) Correctly flags incorrect arguments
**		e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	K  	2
#define	N  	100
#define NS	30
#define MAXSH	32
#define SHSTEP	4
#define NUMSH	MAXSH/SHSTEP+1
#define	NARGS	5

void 
Tstdiir1(char *rcode)
{
    short i, j, k, errc;
    short output[N];
    long coeff2[6 * K];
    int rshift, nsect;
    long *work, sum;

#include "tstdiir.txt"
#include "filtdat.txt"
    static int stkchk[NARGS];

    if (InitDIir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

/* test working of stack */
    nsect = K;
    stkchk[0] = (int)output;
    stkchk[1] = input[0];
    stkchk[2] = (int)coeff;
    stkchk[3] = nsect;
    stkchk[4] = (int)work;

    /* also test for OK return argument */
    if (DIir1(output, input[0], coeff, nsect, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");

    if (stkchk[0] != (int)output ||
	stkchk[1] != input[0] ||
	stkchk[2] != (int)coeff ||
	stkchk[3] != nsect ||
	stkchk[4] != (int)work)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 - run through remaining NS-1 samples */
    for (i = 1; i < NS; i++)
	DIir1(output + i, input[i], coeff, nsect, work);
    errc = 0;
    for (i = 0; i < NS; i++)
    {
	if (output[i] != t1[i])
	    errc++;
    }
    if (errc)
	sprintf(rcode,"Failed Preliminary Test: %d errors\n", errc);
    free(work);

/* test return args */
    if (InitDIir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");
    for (nsect = -2; nsect <= 0; nsect++)
	if (DIir1(output, input[0], coeff, nsect, work) != EDSP_BAD_ARG)
	    sprintf(rcode,"Bad Return Argument - nsect\n");
    for (i = 0; i < 6 * K; i++)
	coeff2[i] = coeff[i];
    /* test first section */
    for (rshift = -2; rshift < 0; rshift++)
    {
	coeff2[0] = rshift;
	if (DIir1(output, input[0], coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode,"Bad Return Argument - rshift\n");
    }
    for (rshift = 33; rshift < 35; rshift++)
    {
	coeff2[0] = rshift;
	if (DIir1(output, input[0], coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode,"Bad Return Argument - rshift\n");
    }
    /* test second section */
    coeff2[0] = 0;
    for (rshift = -2; rshift < 0; rshift++)
    {
	coeff2[6] = rshift;
	if (DIir1(output, input[0], coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode,"Bad Return Argument - rshift\n");
    }
    for (rshift = 49; rshift < 50; rshift++)
    {
	coeff2[6] = rshift;
	if (DIir1(output, input[0], coeff2, K, work) != EDSP_BAD_ARG)
	    sprintf(rcode,"Bad Return Argument - rshift\n");
    }
    free(work);

/*	filter sinusoids at frequencies up to 0.5*Fs in steps of 0.05*Fs
	and compare with precalculated data
	NB: The test data is generated using doubles which does not have enough
	precision to accurately model filtering of the largest frequency
	sinusoid (0.5*Fs,k=N/2) compared with a 64 bit accumulator.
	This sinusoid is not included.
*/
    j = 0;
    errc = 0;
    for (k = 0; k < N / 2; k += N / 20)
    {
	for (rshift = 0; rshift <= MAXSH; rshift += SHSTEP)
	{
	    if (InitDIir(&work, K) != EDSP_OK)
		sprintf(rcode, "Init Problem\n");
	    coeff[0] = rshift;
	    coeff[6] = rshift + 16;
	    for (i = 0; i < N; i++)
		DIir1(output + i, testdat[j][i], coeff, K, work);;
	    sum = 0;
	    for (i = 0; i < N; i++)
		sum += output[i];
	    if (sum != sumdiir[j][rshift / SHSTEP])
		errc++;
	    free(work);
	}
	j++;
    }
    if (errc)
	sprintf(rcode,"Failed Sinusoid Test: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "DIir1 Passed\n");
    else
	strcat(rcode, "DIir1 Failed\n");
}
