/*
** FILE NAME:   tstfir.c
**
** PROJECT ID:  94-0040
**
** TITLE:       File that tests the DSPLIB Fir() function
**
** AUTHOR:      Ensigma Ltd.
**
** DESCRIPTION: The function Tstfir() checks that Fir():
**              a) Does not corrupt the stack
**              b) Passes a simple test vector
**              c) Handles small array sizes
**              d) Correctly flags incorrect arguments
**              e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"


#define K       12
#define N       100
#define MAXSH   27
#define NARGS   7

void 
Tstfir(char *rcode)
{
    short *work, i, j, k, errc;
    short output[N];
    int nsamp, ncoeff, rshift,n;
    long sum;
	#include "tstfir.txt"
	#include "filtdat.txt"

    static int stkchk[NARGS];

    /** My Prog Here **/
    for(n=0; n<N; n++)
    output[n] = 1234;




/* test vecs - filter cutoff (-6dB) at 0.25*Fs */

    if (InitFir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

/* test working of stack */
    nsamp = 10;
    ncoeff = K;
    rshift = 0;
    stkchk[0] = (int)output;
    stkchk[1] = (int)input;
    stkchk[2] = nsamp;
    stkchk[3] = (int)coeff;
    stkchk[4] = ncoeff;
    stkchk[5] = rshift;
    stkchk[6] = (int)work;

    printf("Begin FIR");

    /** also test for OK return argument **/
    if (Fir(output, input, nsamp, coeff, ncoeff, rshift, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    
    printf("End FIR");
    
    
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)input ||
	stkchk[2] != nsamp ||
	stkchk[3] != (int)coeff ||
	stkchk[4] != ncoeff ||
	stkchk[5] != rshift ||
	stkchk[6] != (int)work)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

    printf("End Of Stack Check");

/* continue with test vector 1  - tests for correct workspace usage */
    if (Fir(output+nsamp, input+nsamp, N-nsamp, coeff, ncoeff, rshift, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    free(work);
    errc = 0;
    for (i = 0; i < N; i++)
	if (output[i] != t1[i])
	    errc++;
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    if (InitFir(&work, K) != EDSP_OK)
	sprintf(rcode,"Init Problem\n");
    for (nsamp = -2; nsamp <= 0; nsamp++)
	if (Fir(output, input, nsamp, coeff, K, 0, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - nsamp=%d\n", nsamp);
    for (ncoeff = -2; ncoeff <= 2; ncoeff++)
	if (Fir(output, input, N, coeff, ncoeff, 0, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - ncoeff=%d\n", ncoeff);
    for (rshift = -2; rshift < 0; rshift++)
	if (Fir(output, input, N, coeff, K, rshift, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift=%d\n", rshift);
    for (rshift = 28; rshift < 45; rshift++)
	if (Fir(output, input, N, coeff, K, rshift, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument - rshift=%d\n", rshift);
    free(work);

/* test small array sizes */
    /* small number of inputs */
    for (i = 0; i < N; output[i++] = 0) ;
    ncoeff = K;
    rshift = 0;
    errc = 0;
    for (nsamp = 1; nsamp < 5; nsamp++)
    {
	if (InitFir(&work, K) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	Fir(output, input, nsamp, coeff, ncoeff, rshift, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T1: %d errors\n", errc);

    /* small number of coefficients */
    for (i = 0; i < N; output[i++] = 0) ;
    rshift = 0;
    errc = 0;
    for (ncoeff = 3; ncoeff < 7; ncoeff++)
    {
	nsamp = ncoeff;
	if (InitFir(&work, ncoeff) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	Fir(output, input, nsamp, coeff, ncoeff, rshift, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T2: %d errors\n", errc);

/*      filter sinusoids at frequencies up to 0.5*Fs in steps of 0.05*Fs
	and compare with precalculated data */
    j = 0;
    errc = 0;
    for (k = 0; k <= N / 2; k += N / 20)
    {
	for (rshift = 0; rshift <= MAXSH; rshift++)
	{
	    if (InitFir(&work, K) != EDSP_OK)
		sprintf(rcode, "Init Problem\n");
	    Fir(output, testdat[j], N, coeff, K, rshift, work);
	    sum = 0;
	    for (i = 0; i < N; i++)
		sum += output[i];
	    if (sum != sumfir[j][rshift])
		errc++;
	    free(work);
	}
	j++;
    }
    if (errc)
	sprintf(rcode, "Failed Sinusoid Test: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Fir Passed\n");
    else
	strcat(rcode, "Fir Failed\n");
}
