/*
** FILE NAME:	tstfir1.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB Fir1() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tstfir1() checks that Fir1():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Handles small array sizes
**		d) Correctly flags incorrect arguments
**		e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	K  	12
#define	N  	100
#define MAXSH	27
#define	NARGS	6

/* FIR1 Test Program */
void 
Tstfir1(char *rcode)
{
    short *work, i, j, k, errc;
    short output[N];
    int ncoeff, rshift;
    long sum;

    static int stkchk[NARGS];

/* test vecs - filter cutoff (-6dB) at 0.25*Fs */
#include "filtdat.txt"
#include "tstfir.txt"

    if (InitFir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

/* test working of stack */
    ncoeff = K;
    rshift = 0;
    stkchk[0] = (int)output;
    stkchk[1] = input[0];
    stkchk[2] = (int)coeff;
    stkchk[3] = ncoeff;
    stkchk[4] = rshift;
    stkchk[5] = (int)work;

    /* also test for OK return argument */
    if (Fir1(output, input[0], coeff, ncoeff, rshift, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != input[0] ||
	stkchk[2] != (int)coeff ||
	stkchk[3] != ncoeff ||
	stkchk[4] != rshift ||
	stkchk[5] != (int)work)
    {

	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 - run through remaining N-1 samples */
    for (i = 1; i < N; i++)
	Fir1(output + i, input[i], coeff, ncoeff, rshift, work);
    errc = 0;
    for (i = 0; i < N; i++)
	if (output[i] != t1[i])
	    errc++;
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);
    free(work);

/* test return args */
    if (InitFir(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");
    for (ncoeff = -2; ncoeff <= 2; ncoeff++)
	if (Fir1(output, input[0], coeff, ncoeff, 0, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = -2; rshift < 0; rshift++)
	if (Fir1(output, input[0], coeff, K, rshift, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    for (rshift = 28; rshift < 45; rshift++)
	if (Fir1(output, input[0], coeff, K, rshift, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");
    free(work);

/* test small array sizes */
    /* small number of coefficients */
    for (i = 0; i < N; output[i++] = 0) ;
    rshift = 0;
    errc = 0;
    for (ncoeff = 3; ncoeff < 7; ncoeff++)
    {
	if (InitFir(&work, ncoeff) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	for (i = 0; i < ncoeff; i++)
	    Fir1(output + i, input[i], coeff, ncoeff, rshift, work);
	if (output[ncoeff - 1] != t1[ncoeff - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T1: %d errors\n", errc);

/*	filter sinusoids at frequencies up to 0.5*Fs in steps of 0.05*Fs
	and compare with precalculated data */
    j = 0;
    errc = 0;
    for (k = 0; k <= N / 2; k += N / 20)
    {
	for (rshift = 0; rshift <= MAXSH; rshift++)
	{
	    if (InitFir(&work, K) != EDSP_OK)
		sprintf(rcode, "Init Problem\n");
	    for (i = 0; i < N; i++)
		Fir1(output + i, testdat[j][i], coeff, K, rshift, work);
	    sum = 0;
	    for (i = 0; i < N; i++)
		sum += output[i];
	    if (sum != sumfir[j][rshift])
		errc++;
	    free(work);
	}
	j++;
    }
    if (errc)
	sprintf(rcode, "Failed Sinusoid Test: %d errors\n", errc);

    if (strlen(rcode) == 0)
	sprintf(rcode, "Fir1 Passed\n");
    else
	strcat(rcode, "Fir1 Failed\n");
}
