/*
** FILE NAME:	tstlms.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB Lms() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tstlms() checks that Lms():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Handles small array sizes
**		d) Correctly flags incorrect arguments
**		e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	K  	12
#define	N  	200
#define	TWOMU	32767
#define	RSHIFT	15
#define NARGS	9
#define MAXSH	27

void 
Tstlms(char *rcode)
{
    short *work, i, errc;
    short output[N], h[K];
    short twomu;
    int nsamp, ncoeff, rshift;

    static int stkchk[NARGS];

#include "tstlms.txt"

    if (InitLms(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

    for (i = 0; i < K; i++)
	h[i] = hinit[i];

/* test working of stack */
    nsamp = 10;
    ncoeff = K;
    rshift = RSHIFT;
    twomu = TWOMU;
    stkchk[0] = (int)output;
    stkchk[1] = (int)noise;
    stkchk[2] = (int)ref;
    stkchk[3] = nsamp;
    stkchk[4] = (int)h;
    stkchk[5] = ncoeff;
    stkchk[6] = rshift;
    stkchk[7] = twomu;
    stkchk[8] = (int)work;

    /* also test for OK return argument */
    if (Lms(output, noise, ref, nsamp, h, ncoeff, rshift, twomu, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != (int)noise ||
	stkchk[2] != (int)ref ||
	stkchk[3] != nsamp ||
	stkchk[4] != (int)h ||
	stkchk[5] != ncoeff ||
	stkchk[6] != rshift ||
	stkchk[7] != twomu ||
	stkchk[8] != (int)work)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* continue with test vector 1  - tests for correct workspace usage */
    if (Lms(output+nsamp, noise+nsamp, ref+nsamp, N-nsamp, h, ncoeff, rshift, twomu, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    free(work);
    errc = 0;
    for (i = 0; i < N; i++)
	if (output[i] != t1[i])
	    errc++;
    for (i = 0; i < K; i++)
	if (h[i] != t2[i])
	    errc++;
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);

/* test return args */
    if (InitLms(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");
    for (nsamp = -2; nsamp <= 0; nsamp++)
	if (Lms(output, noise, ref, nsamp, h, K, RSHIFT, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (nsamp=%d) \n", nsamp);
    for (ncoeff = -2; ncoeff <= 2; ncoeff++)
	if (Lms(output, noise, ref, N, h, ncoeff, RSHIFT, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (ncoeff=%d)\n", ncoeff);
    for (rshift = -2; rshift < 0; rshift++)
	if (Lms(output, noise, ref, N, h, K, rshift, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (rshift=%d)\n", rshift);
    for (rshift = 28; rshift < 45; rshift++)
	if (Lms(output, noise, ref, N, h, K, rshift, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (rshift=%d)\n", rshift);
    free(work);

/* test small array sizes */
    /* small number of inputs */
    for (i = 0; i < N; output[i++] = 0) ;
    errc = 0;
    for (nsamp = 1; nsamp < K; nsamp++)
    {
	if (InitLms(&work, K) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	for (i = 0; i < K; i++)
	    h[i] = hinit[i];
	Lms(output, noise, ref, nsamp, h, K, RSHIFT, TWOMU, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T1: %d errors\n", errc);

    /* small number of coefficients */
    for (i = 0; i < N; output[i++] = 0) ;
    errc = 0;
    for (ncoeff = 3; ncoeff < 7; ncoeff++)
    {
	if (InitLms(&work, ncoeff) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	nsamp = ncoeff;
	for (i = 0; i < K; i++)
	    h[i] = hinit[i];
	Lms(output, noise, ref, nsamp, h, ncoeff, RSHIFT, TWOMU, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T2: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Lms Passed\n");
    else
	strcat(rcode, "Lms Failed\n");
}
