/*
** FILE NAME:	tstlms1.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB Lms1() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tstlms1() checks that Lms1():
**		a) Does not corrupt the stack
**		b) Passes a simple test vector
**		c) Handles small array sizes
**		d) Correctly flags incorrect arguments
**		e) Passes a bit exact sinusoid filtering test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"

#define	K  	12
#define	N  	200
#define	TWOMU	32767
#define	RSHIFT	15
#define NARGS	8
#define MAXSH	27

void 
Tstlms1(char *rcode)
{
    short *work, i, errc;
    short output[N], h[K];
    short twomu;
    int nsamp, ncoeff, rshift;

    static int stkchk[NARGS];

#include "tstlms.txt"

    if (InitLms(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");

    for (i = 0; i < K; i++)
	h[i] = hinit[i];

/* test working of stack */
    nsamp = N;
    ncoeff = K;
    rshift = RSHIFT;
    twomu = TWOMU;
    stkchk[0] = (int)output;
    stkchk[1] = noise[0];
    stkchk[2] = ref[0];
    stkchk[3] = (int)h;
    stkchk[4] = ncoeff;
    stkchk[5] = rshift;
    stkchk[6] = twomu;
    stkchk[7] = (int)work;

    /* also test for OK return argument */

    if (Lms1(output, noise[0], ref[0], h, ncoeff, rshift, twomu, work) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)output ||
	stkchk[1] != noise[0] ||
	stkchk[2] != ref[0] ||
	stkchk[3] != (int)h ||
	stkchk[4] != ncoeff ||
	stkchk[5] != rshift ||
	stkchk[6] != twomu ||
	stkchk[7] != (int)work)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test vector 1 */
    errc = 0;
    for (i = 1; i < N; i++)
    {
	if (Lms1(output + i, noise[i], ref[i], h, ncoeff, rshift, twomu, work) != EDSP_OK)
	    sprintf(rcode, "EDSP_OK not returned\n");
	if (output[i] != t1[i])
	    errc++;
    }
    for (i = 0; i < K; i++)
	if (h[i] != t2[i])
	    errc++;
    if (errc)
	sprintf(rcode, "Failed Preliminary Test: %d errors\n", errc);
    free(work);

/* test return args */
    if (InitLms(&work, K) != EDSP_OK)
	sprintf(rcode, "Init Problem\n");
    for (ncoeff = -2; ncoeff <= 2; ncoeff++)
	if (Lms1(output, noise[0], ref[0], h, ncoeff, RSHIFT, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (ncoeff=%d)\n", ncoeff);
    for (rshift = -2; rshift < 0; rshift++)
	if (Lms1(output, noise[0], ref[0], h, K, rshift, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (rshift=%d)\n", rshift);
    for (rshift = 28; rshift < 45; rshift++)
	if (Lms1(output, noise[0], ref[0], h, K, rshift, TWOMU, work) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument (rshift=%d)\n", rshift);
    free(work);

/* test small array sizes */
    /* small number of coefficients */
    for (i = 0; i < N; output[i++] = 0) ;
    errc = 0;
    for (ncoeff = 3; ncoeff < 7; ncoeff++)
    {
	if (InitLms(&work, ncoeff) != EDSP_OK)
	    sprintf(rcode, "Init Problem\n");
	nsamp = ncoeff;
	for (i = 0; i < K; i++)
	    h[i] = hinit[i];
	for (i = 0; i < nsamp; i++)
	    Lms1(output + i, noise[i], ref[i], h, ncoeff, RSHIFT, TWOMU, work);
	if (output[nsamp - 1] != t1[nsamp - 1])
	    errc++;
	free(work);
    }
    if (errc)
	sprintf(rcode, "Failed T2: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Lms1 Passed\n");
    else
	strcat(rcode, "Lms1 Failed\n");
}
