/*
** FILE NAME:   tstwin1.c
**
** PROJECT ID:  n/a
**
** TITLE:       File that tests the DSPLIB window functions
**
** AUTHOR:      n/a
**
** DESCRIPTION: The function Tstwin() checks that each of the functions
**              GenBlackman(), GenHanning(), GenHamming() and GenTriangle()
**              a) Correctly flags incorrect arguments
**              b) Tests the floating point library against precalculated data
**              c) Tests the quantisation noise is within tolerance
**              d) Passes a simple test vector for bit-exactness (within tol.)
**
** PARAMETERS:
**              char *rcode     : pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <ensigdsp.h>
#include "testhead.h"

#ifndef DBL_EPSILON             /* account for alternative DBL_EPSILON */
#define DBL_EPSILON     DBL_POS_EPS
#endif

#define FAIL    0
#define PASS    1
#define SC      32767
#define MAXN    2048            /* maximum size of window */
#define NTEST   32              /* length of precalc'd test */
#define NRTEST  NTEST-1         /* length of rounding test */
#define PREC    100.0*SC*DBL_EPSILON    /* suitable machine accuracy */
#define FIXERR  10*log10(1.0/12)+0.75   /* (dB) variance of error = delta^2/12,
					 * allow a */
 /* 0.75 dB tolerance due to noise correlation */
#define FLTERR  -65             /* (dB) suitable threshold for floating point
				 * calcs */
#define TWOPI   6.283185307179
	float   epsilon;
	float   dave;
	double   error;
	float   poseps;
	double DHanning;
/*
** FUNCTION:    validate1
**
** DESCRIPTION: Check tolerance of floating point function
**
*/

int 
Hanning()
{
    DHanning = (SC * 0.5 * (1.0 - cos(TWOPI * 7 / 28)));
}

/*
** FUNCTION:    roundtest
**
** DESCRIPTION: Determine the rounding (quantisation) noise of the fixed point
**              window function in comparison with a floating point version.
**
*/
double 
roundtest(int len, int (*fixpnt) (short *out, long n), double (*flpnt) (int n, int len))
{
    short output[MAXN];
    int n;
    double temp, err = 0;

    fixpnt(output, len);
    for (n = 0; n < len; n++)
    {
	temp = flpnt(n, len);
	err += (output[n] - temp) * (output[n] - temp);
    }
    return (10 * log10(err / len));
}

void 
Tstwin(char *rcode)
{
    short output[MAXN];
    long len, n;
    double temp, err1, err2;
	/* check small window lengths on a bit by bit basis
   NB a tolerance of 0.5 bit is permitted (within machine accuracy) */

    len = 28;
    
	
	GenHanning(output, len);
	n = 7;
	 
	 Hanning();
		error = (fabs(DHanning - output[n]));
	    if ((fabs(DHanning - output[n])) > (0.5 + PREC))
		printf(" Hanning Failed ");
	 
	printf("End Of File");
}


