/*
** FILE NAME:	tvar.c
**
** PROJECT ID:	94-0040
**
** TITLE:	File that tests the DSPLIB Variance() function
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	The function Tvar() checks that Variance()
**		a) Does not corrupt the stack
**		b) Correctly flags incorrect arguments
**		c) Passes a bit-exact test
**
** PARAMETERS:
**		char *rcode	: pointer to return pass/fail information
**
** MODIFICATION HISTORY:
**
**              1.00    15-Sep-95       Initial Revision
**
*/

#include <stdio.h>
#include <string.h>
#include <ensigdsp.h>
#include "testhead.h"
#include "miscdat.txt"
#include "tvar.txt"

#define	N	100
#define	NARGS	4

void 
Tvar(char *rcode)
{
    long size, var[1];
    short mean[1];
    int i, errc;

    static int stkchk[NARGS];



/* test working of stack */
    size = N;
    stkchk[0] = (int)var;
    stkchk[1] = (int)mean;
    stkchk[2] = (int)dat;
    stkchk[3] = size;

    /* also test for OK return argument */
    if (Variance(var, mean, dat, size) != EDSP_OK)
	sprintf(rcode, "EDSP_OK not returned\n");
    if (stkchk[0] != (int)var ||
	stkchk[1] != (int)mean ||
	stkchk[2] != (int)dat ||
	stkchk[3] != size)
    {
	sprintf(rcode, "Function arguments altered - aborting");
	exit(1);
    }

/* test return args */
    for (size = -2; size < 1; size++)
	if (Variance(var, mean, dat, size) != EDSP_BAD_ARG)
	    sprintf(rcode, "Bad Return Argument\n");

    errc = 0;
    for (i = 0; i < N; i++)
    {
	if (Variance(var, mean, dat, i + 1) != EDSP_OK)
	    sprintf(rcode, "EDSP_OK not returned\n");
	if (*var != outvar[i])
	    errc++;
	if (*mean != outmean[i])
	    errc++;
    }
    if (errc)
	sprintf(rcode, "Failed: %d errors\n", errc);
    if (strlen(rcode) == 0)
	sprintf(rcode, "Variance Passed\n");
    else
	strcat(rcode, "Variance Failed\n");
}
