;
; FILE NAME:    variance.asm
;
; PROJECT ID:   94-0040
;
; TITLE:        Variance
;
; AUTHOR:       Ensigma Ltd.
;
; DESCRIPTION:  This file contains the variance function.
;

	.EXPORT	_Variance

;REGISTER USAGE
;
;R4 -> VARIANCE OUTPUT
;R5 -> MEAN OUTPUT
;R6 -> INPUT
;R7 -> NO

	.SECTION	P,CODE,ALIGN=4

; Preserve registers on stack, load stack parameters and perform error checking
_Variance:	MOV.L	R14,@-R15	;PUSH REGISTERS ON STACK
	MOV.L	R13,@-R15
	MOV.L	R12,@-R15
	MOV.L	R11,@-R15
	MOV.L	R10,@-R15
	MOV.L	R9,@-R15
	MOV.L	R8,@-R15
	STS.L	MACH,@-R15
	STS.L	MACL,@-R15
	STS.L	PR,@-R15

;ERROR CHECKING
	CMP/PL	R7		;NO >0 ?
	BT	argOK

	BRA	RTN
	MOV	#1,R0		;R0= RETURN CODE

	.ALIGN	4
argOK:	MOV	R7,R8
	SHLL	R8
	ADD	R6,R8		;R8=LAST DATA ADDRESS + 2

	MOV	R6,R10		;R10=R6=INPUT ADDRESS

	MOV	UPMASK,R11	;MASK AWAY UPPER 30 BITS
	AND	R7,R11		;R11=LOWER 2 BITS

	MOVA	NX,R0
	SHLL2	R11		;R11*4
	SUB	R11,R0		;R0=ADDRESS OF LB - R11*2
	JMP	@R0		;R0=JUMP ADDRESS
	MOV	#0,R1

; First calculate mean
	.ALIGN	4
mloop:	MOV.W	@R6+,R2
	ADD	R2,R1
	MOV.W	@R6+,R2
	ADD	R2,R1
	MOV.W	@R6+,R2
	ADD	R2,R1
	MOV.W	@R6+,R2
	ADD	R2,R1
NX:	CMP/EQ	R6,R8
	BF	mloop

	MOV	R7,R9
	SHLR	R9

DIVI:	MOV	R1,R13		;SAVE SIGN
	CMP/PZ	R1
	BT	DIVI1
	SUB	R9,R1		;ADD ROUNDING FOR -VE NO
	BRA	DIVI2
	NEG	R1,R1

DIVI1:	ADD	R9,R1		;ADD ROUNDING FOR +VE NO
DIVI2:	MOV	R7,R11		;R11=R7=NO
	SHLL16	R11		;UPPER 16 BITS=DIVISOR
	DIV0U			;FLAG INITIALISATION
	.arepeat	16
	DIV1	R11,R1		;REPEAT 16 TIMES
	.aendr
	ROTCL	R1
	EXTU.W	R1,R2		;R2=QUOTIENT=MEAN

	CMP/PZ	R13		;IS SIGN POSITIVE ?
	BT	OUTPUT
	NEG	R2,R2
OUTPUT:	MOV.W	R2,@R5		;OUTPUT MEAN


	MOV	#0,R13		;R12:R13=SUM=0
	MOV	#0,R12
	MOV	#0,R3
	MOV.L	MIN,R11		;R11=H'FFFF8000
	MOV.L	MAX,R9		;R9=H'00008000

; Now calculate variance
	CLRT
vloop:	MOV.W	@R10+,R1	;READ X(I)
	SUB	R2,R1		;X(I)-MEAN

	CMP/GT	R1,R11
	BT	NEGOV		;NEGATIVE OVERFLOW
	CMP/GT	R9,R1
	BT	POSOV		;POSITIVE OVERFLOW

	MULS	R1,R1		;SQUARE IT
	STS	MACL,R1
	ADDC	R1,R13		;SUM IT
	ADDC	R3,R12		;ADD CARRY
	CMP/EQ	R10,R8
	BF	vloop
	BRA	RND1
	MOV	R7,R9

; Overflow correction code
NEGOV:	SUB	R11,R1		;R1-H'FFFF8000
	MOV	R1,R14		;R14=R1
	MULS	R1,R1		;SQUARE R1
	STS	MACL,R1
	SHLL16	R14
	CMP/PZ	R14
	BT	NEGOV1
	NEG	R14,R14		;TAKE ABSOLUTE VALUE OF R14
NEGOV1:	ADD	R14,R1		;ADD R14 << 16
	MOV	SQMAX,R14
	ADD	R14,R1		;ADD MAX*MAX=H'40000000
	CLRT			;CLEAR T
	ADDC	R1,R13		;SUM IT
	ADDC	R3,R12		;ADD CARRY
	CMP/EQ	R10,R8
	BF	vloop
	BRA	RND1
	MOV	R7,R9

POSOV:	SUB	R9,R1		;SUB H'8000
	MOV	R1,R14		;R14=R1
	MULS	R1,R1		;SQUARE R1
	STS	MACL,R1
	SHLL16	R14
	CMP/PZ	R14
	BT	POSOV1
	NEG	R14,R14		;TAKE ABSOLUTE VALUE OF R14
POSOV1:	ADD	R14,R1		;ADD R14 << 16
	MOV	SQMAX,R14
	ADD	R14,R1		;ADD MAX*MAX=H'40000000
	CLRT			;CLEAR T
	ADDC	R1,R13		;SUM IT
	ADDC	R3,R12		;ADD CARRY
	CMP/EQ	R10,R8
	BF	vloop


; Rounding code
RND:	MOV	R7,R9
RND1:	SHLR	R9
	CLRT
	ADDC	R9,R13		;ADD ROUNDING
	ADDC	R3,R12

	DIV0U			;FLAG INITIALISATION
	.arepeat	32
	ROTCL	R13		;REPEAT 32 TIMES
	DIV1	R7,R12
	.aendr
	ROTCL	R13		;R13=QUOTIENT

; Have now calculated the variance, so store the result
	MOV.L	R13,@R4		;OUTPUT VARIANCE

	MOV	#0,R0		;RETURN CODE
RTN:	LDS.L	@R15+,PR
	LDS.L	@R15+,MACL
	LDS.L	@R15+,MACH
	MOV	@R15+,R8
	MOV	@R15+,R9
	MOV	@R15+,R10
	MOV	@R15+,R11
	MOV	@R15+,R12
	MOV	@R15+,R13
	RTS
	MOV	@R15+,R14

	.ALIGN	4
UPMASK	.data.l	H'00000003
MIN	.data.l	H'FFFF8000
MAX	.data.l	H'00008000
SQMAX	.data.l	H'40000000

	.END
