/*
** FILE NAME:	variance.c
**
** PROJECT ID:	94-0040
**
** TITLE:	Variance
**
** AUTHOR:	Ensigma Ltd.
**
** DESCRIPTION:	This routine calculates the variance and mean of input data.
**
**		int Variance(variance, mean, input, no_elements)
**
** PARAMETERS:
**              long *variance       : variance of data
**              short *mean          : mean of data
**              short input[]        : array of input data
**              long no_elements     : size of the input buffer
**
** RETURNS:     EDSP_OK                : success
**              EDSP_BAD_ARG           : no_elements < 1
**
** MODIFICATION HISTORY:
**              1.0  15-9-95    Initial revision
*/

#include <math.h>
#include <ensigdsp.h>

int
Variance(long *variance,
	 short *mean,
	 const short input[],
	 long no_elements)
{
    /* Variable Declaration */
    int i;
    double temp;

    /* Input Argument Error checking */
    if (no_elements < 1)
	return EDSP_BAD_ARG;

    temp = 0;

    if (Mean(mean, input, no_elements) != EDSP_OK)	/* Mean procedure call */
	return EDSP_BAD_ARG;

    for (i = 0; i < no_elements; i++)	/* Main Loop */
	temp += ((double)input[i] - *mean) * ((double)input[i] - *mean);

    /* Rounding */
    if (temp < 0)
	*variance = ceil(temp / no_elements - 0.5);
    else
	*variance = floor(temp / no_elements + 0.5);

    return EDSP_OK;		/* Return success */
}
